'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.noop = noop;
exports.bindState = bindState;
exports.update = update;
exports.updated = updated;
exports.buildFormValidator = buildFormValidator;

var _lodash = require('lodash.set');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function noop() {}

function bindState(component) {
  var key = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'form';

  return {
    attrs: component.state && component.state[key] || {},
    onChange: function onChange(attrs) {
      return component.setState(_defineProperty({}, key, attrs));
    }
  };
}

function update(obj, name, value) {
  _update(obj, name, value);

  return obj;
}

function updated(obj, name, value) {
  var current = _extends({}, obj);

  return update(current, name, value);
}

function _update(current, name, value) {
  var match = name.match(/^([\w\d]+)\.?(.+)?$/);
  var key = match[1],
      rest = match[2];


  if (current[key] === undefined) {
    return (0, _lodash2.default)(current, name.split('.'), value);
  }
  if (!rest) {
    return current[key] = value;
  }

  current[key] = Array.isArray(current[key]) ? [].concat(_toConsumableArray(current[key])) : _extends({}, current[key]);
  _update(current[key], rest, value);
}

function wildcard(name) {
  return name.replace(/\d+/g, '*');
}

function buildFormValidator(form) {
  function validate(name) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var value = options.hasOwnProperty('value') ? options.value : form.get(name);
    var validator = options['with'] || form._validations[name] || form._validations[wildcard(name)];

    if (!validator) return null;

    var error = callValidator(validator);

    if (error) {
      return validate.addError(name, error);
    }

    return null;

    function callValidator(validator) {
      if (Array.isArray(validator)) {
        return callArrayValidator(validator);
      }
      if (typeof validator === 'string') {
        return callStringValidator(validator);
      }
      if (typeof validator === 'function') {
        return validator.call(form, value);
      }
      if (validator && (typeof validator === 'undefined' ? 'undefined' : _typeof(validator)) === 'object') {
        return callObjectValidator(validator);
      }
      throw new Error('unable to use \'' + validator + '\' as validator function');
    }

    function stringToValidator(name) {
      var validator = form.constructor.validations[name];

      if (!validator) throw new Error(name + ' validation rule is not defined');

      return validator;
    }

    function callStringValidator(name, options) {
      return stringToValidator(name).call(form, value, options === true ? {} : options);
    }

    function callObjectValidator(obj) {
      for (var _name in obj) {
        var _error = callStringValidator(_name, obj[_name]);
        if (_error) return _error;
      }
    }

    function callArrayValidator(ary) {
      for (var i = 0; i < ary.length; i++) {
        var _error2 = callValidator(ary[i]);
        if (_error2) return _error2;
      }
    }
  }

  Object.assign(validate, {
    errors: {},
    clearErrors: function clearErrors() {
      this.errors = {};
    },
    addError: function addError(key, message) {
      this.errors[key] = message;
      return message;
    },
    nested: function nested(ref) {
      var errors = form.refs[ref].performValidation();

      if (Object.getOwnPropertyNames(errors).length > 0) {
        this.addError(ref, 'invalid');
      }
    }
  });

  return validate;
}